"""GGPXPolyline for GPX LogFile type"""

__author__  = '''774RR <774@hogehogehogejohn.doe>'''
__docformat__ = 'plaintext'



try:
  from Products.LinguaPlone.public import *
except ImportError:
  # No multilingual support
  from Products.Archetypes.public import *

#for debug
from Products.Archetypes.debug import log

from AccessControl import ClassSecurityInfo
from OFS.Image import Pdata
from OFS.Image import File as OFSFile
from OFS.Image import File as OFSImage

from Products.CMFCore.permissions import View
from Products.ATContentTypes.content.base import ATCTContent
from Products.ATContentTypes.content.schemata import ATContentTypeSchema
from Products.ATContentTypes.lib.historyaware import HistoryAwareMixin
from Products.ATContentTypes.configuration import zconf
from Products.ATContentTypes.content.document import ATDocumentSchema
from Products.ATContentTypes.content.document import ATDocument
from Products.CMFPlone.interfaces.NonStructuralFolder import INonStructuralFolder

from Products.ATContentTypes.content.schemata import finalizeATCTSchema

try:
    from Products.FileSystemStorage.FileSystemStorage import FileSystemStorage
except:
    raise DependancyError('If you want to use FileSystemStorage you need to install the FileSystemStorage Product')

try:
    from Products.validation.validators.SupplValidators import MaxSizeValidator
    HAS_MAXSIZE_VALIDATOR = True
except ImportError:
    HAS_MAXSIZE_VALIDATOR = False
    pass

from Products.ATGoogleMaps.config import *
from Products.ATGoogleMaps.lib.creategraph import CreateGraph
from Products.ATGoogleMaps.lib.gpxanalyze import GPXAnalyze
from cStringIO import StringIO

GGPXPolylineSchema = ATContentTypeSchema.copy() + Schema((


    FileField('gpxlogfile',
        required=1,
        primary=1,
        validators=MaxSizeValidator("checkFileMaxSize", maxsize=MAX_FILE_SIZE),
        storage = FileSystemStorage(),
        widget=FileWidget(
            label="GPX Log File",
            label_msgid="label_gpxlogfile",
            description=' Drow polyline in Google Maps (GPX(the GPS eXchange format) file only!).',
            description_msgid="help_gpxlogfile",
            i18n_domain="googlemaps"
        )
    ),

    TextField('detail',
        searchable=True,
        validators=('isTidyHtmlWithCleanup',),
        default_content_type=zconf.ATDocument.default_content_type,
        default_output_type='text/x-html-safe',
        allowable_content_types=zconf.ATDocument.allowed_content_types,
        widget=RichWidget(
            rows=10,
            label="Detail",
            label_msgid="label_detail",
            description_msgid="help_detail",
            i18n_domain="googlemaps",
        ),
    ),
    
    StringField('color',
        default='#FF0000',
        widget=StringWidget(
            label='Color',
            label_msgid='label_color',
            description='Enter the color in hexadecimal numeric HTML style, i.e. #RRGGBB.',
            description_msgid='help_color',
            i18n_domain='googlemaps',
            size=10,
        )
    ),

    IntegerField('width',
        default=2,
        vocabulary=range(1,20),
        widget=SelectionWidget(
            label='Width',
            label_msgid='label_width',
            description='Select a width of the line in pixels.',
            description_msgid='help_width',            
            i18n_domain='googlemaps',
        )
    ),
    
    BooleanField('opacity',
        widget=BooleanWidget(
            label='Opacity',
            label_msgid='label_opacity',
            description_msgid='help_opacity',
            i18n_domain='googlemaps',
        )
    ),
    
    BooleanField('viewgraph',
        default=True,
        widget=BooleanWidget(
            label='View Graph',
            label_msgid='label_viewgraph',
            description_msgid='help_viewgraph',
            i18n_domain='googlemaps',
        )
    ),

    # from GPX file
    IntegerField('trk_log_count',
        default=0,
        widget=IntegerWidget(
            label='Track Log Count',
            label_msgid='label_trk_log_count',
            description_msgid='help_trk_log_count',
            i18n_domain='googlemaps',
            visible = {'edit':'invisible', 'view':'visible'},
            modes = ('view',),
        )
    ),

    # from GPX file
    StringField('total_distance',
        default='No Data',
        widget=StringWidget(
            label='Total Distance',
            label_msgid='label_total_distance',
            description_msgid='help_total_distance',
            i18n_domain='googlemaps',
            visible = {'edit':'invisible', 'view':'visible'},
            modes = ('view',),
        )
    ),

    
    # from GPX file
    StringField('start_datetime',
        default='No Data',
        widget=StringWidget(
            label='Start DateTime',
            label_msgid='label_start_datetime',
            description_msgid='help_start_datetime',
            i18n_domain='googlemaps',
            visible = {'edit':'invisible', 'view':'visible'},
            modes = ('view',),
        )
    ),

    # from GPX file
    StringField('end_datetime',
        default='No Data',
        widget=StringWidget(
            label='End DateTime',
            label_msgid='label_end_datetime',
            description_msgid='help_end_datetime',
            i18n_domain='googlemaps',
            visible = {'edit':'invisible', 'view':'visible'},
            modes = ('view',),
        )
    ),

    # from GPX file
    StringField('moveing_time',
        default='No Data',
        widget=StringWidget(
            label='Moving Time',
            label_msgid='label_moveing_time',
            description_msgid='help_moveing_time',
            i18n_domain='googlemaps',
            visible = {'edit':'invisible', 'view':'visible'},
            modes = ('view',),
        )
    ),

    # from GPX file
    StringField('max_speed',
        default='No Data',
        widget=StringWidget(
            label='Max Speed',
            label_msgid='label_max_speed',
            description_msgid='help_max_speed',
            i18n_domain='googlemaps',
            visible = {'edit':'invisible', 'view':'visible'},
            modes = ('view',),
        )
    ),

    # from GPX file
    StringField('avg_speed',
        default='No Data',
        widget=StringWidget(
            label='Average Speed',
            label_msgid='label_avg_speed',
            description_msgid='help_avg_speed',
            i18n_domain='googlemaps',
            visible = {'edit':'invisible', 'view':'visible'},
            modes = ('view',),
        )
    ),

    # from GPX file
    StringField('max_altitude',
        default='No Data',
        widget=StringWidget(
            label='Max Altitude',
            label_msgid='label_max_altitude',
            description_msgid='help_max_altitude',
            i18n_domain='googlemaps',
            visible = {'edit':'invisible', 'view':'visible'},
            modes = ('view',),
        )
    ),

    # from GPX file
    StringField('min_altitude',
        default='No Data',
        widget=StringWidget(
            label='Min Altitude',
            label_msgid='label_min_altitude',
            description_msgid='help_min_altitude',
            i18n_domain='googlemaps',
            visible = {'edit':'invisible', 'view':'visible'},
            modes = ('view',),
        )
    ),

    # from GPX file (auto make)
    ImageField('speed_graph_image',
        widget=ImageWidget(
            label='Distance - Speed Graph Image',
            description='',
            label_msgid='label_speed_graph_image',
            description_msgid='help_speed_graph_image',
            i18n_domain='googlemaps',
            visible = {'edit':'invisible', 'view':'visible'},
            modes = ('view',),
            ),
        sizes= {'preview' : (400, 400),
                'mini'    : (200, 200),
                'thumb'   : (128, 128),
                'tile'    :  (64, 64),
                'icon'    :  (32, 32),
               }
    ),
    
    # from GPX file (auto make)
    ImageField('altitude_graph_image',
        widget=ImageWidget(
            label='Distance - Altitude Graph Image',
            description='',
            label_msgid='label_altitude_graph_image',
            description_msgid='help_altitude_graph_image',
            i18n_domain='googlemaps',
            visible = {'edit':'invisible', 'view':'visible'},
            modes = ('view',),
            ),
        sizes= {'preview' : (400, 400),
                'mini'    : (200, 200),
                'thumb'   : (128, 128),
                'tile'    :  (64, 64),
                'icon'    :  (32, 32),
               }
    ),

),
)

#Finalize schema definition
finalizeATCTSchema(GGPXPolylineSchema)

class GGPXPolyline(ATCTContent, HistoryAwareMixin):
    """Draw GPX Log polyline in Google Maps"""
    
    security = ClassSecurityInfo()

    schema          = GGPXPolylineSchema

    content_icon    = 'ggpxpolyline_icon.gif'
    portal_type     = 'GGPXPolyline'
    meta_type       = 'GGPXPolyline'
    archetype_name  = 'GGPXPolyline'
    default_view    = 'ggpxpolyline_view'
    immediate_view  = 'ggpxpolyline_view'
    suppl_views     = ()
    typeDescription = 'Draw GPX Track Log polyline in Google Maps'
    typeDescMsgId   = 'description_edit_ggpxpolyline'
    global_allow    = 0

    # Get the standard actions (tabs)
    #actions = ATCTContent.actions

    # Make sure we get title-to-id generation when an object is created
    _at_rename_after_creation = True

    def canSetDefaultPage(self):
        """Can this content set default page for folder?"""
        return False

    security.declarePrivate('setTrk_log_count')
    def setTrk_log_count(self, value, cb=None, **kw):
        """set the Track LogCount"""

        if cb or self.isKwStatus( kw, '_initializing_' ) :
            log( "trk_log_count value=%s kw=%s" % (value,kw) )
            f=self.Schema()['trk_log_count']
            return f.set(self, value, **kw)
        return None

    security.declarePrivate('setTotal_distance')
    def setTotal_distance(self,value, cb=None, **kw):
        """set the total distance"""

        f=self.Schema()['total_distance']
        if cb :
            if value >= 1000 :
                value = "%.3f km" % (value / 1000)
            else :
                value = "%d m" % value
            log( "total_distance value=%s kw=%s" % (value,kw) )
            return f.set(self, value, **kw)
        if self.isKwStatus( kw, '_initializing_' ) :
            log( "total_distance value=%s kw=%s" % (value,kw) )
            return f.set(self, value, **kw)
        return None

    security.declarePrivate('setStart_datetime')
    def setStart_datetime(self, value, cb=None, **kw):
        """set the Start DateTime"""

        f=self.Schema()['start_datetime']
        if cb :
            #value = value.strftime("%Y/%m/%d %H:%M:%S" )
            value = "%s" % value
            log( "start_datetime value=%s kw=%s" % (value,kw) )
            return f.set(self, value, **kw)
        if self.isKwStatus( kw, '_initializing_' ) :
            log( "start_datetime value=%s kw=%s" % (value,kw) )
            return f.set(self, value, **kw)
        return None

    security.declarePrivate('setEnd_datetime')
    def setEnd_datetime(self, value, cb=None, **kw):
        """set the End DateTime"""

        f=self.Schema()['end_datetime']
        if cb :
            #value = value.strftime("%Y/%m/%d %H:%M:%S" )
            value = "%s" % value
            log( "end_datetime value=%s kw=%s" % (value,kw) )
            return f.set(self, value, **kw)
        if self.isKwStatus( kw, '_initializing_' ) :
            log( "end_datetime value=%s kw=%s" % (value,kw) )
            return f.set(self, value, **kw)
        return None

    security.declarePrivate('setMoveing_time')
    def setMoveing_time(self, value, cb=None, **kw):
        """set the Moving Time"""

        f=self.Schema()['moveing_time']
        if cb :
            value = "%s" % value
            log( "moveing_time value=%s kw=%s" % (value,kw) )
            return f.set(self, value, **kw)
        if self.isKwStatus( kw, '_initializing_' ) :
            log( "moveing_time value=%s kw=%s" % (value,kw) )
            return f.set(self, value, **kw)
        return None

    security.declarePrivate('setMax_speed')
    def setMax_speed(self, value, cb=None, **kw):
        """set the MAX speed"""

        f=self.Schema()['max_speed']
        if cb :
            value = "%.3f km/h" % value
            log( "max_speed value=%s kw=%s" % (value,kw) )
            return f.set(self, value, **kw)
        if self.isKwStatus( kw, '_initializing_' ) :
            log( "max_speed value=%s kw=%s" % (value,kw) )
            return f.set(self, value, **kw)
        return None

    security.declarePrivate('setAvg_speed')
    def setAvg_speed(self, value, cb=None, **kw):
        """set the AVG speed"""

        f=self.Schema()['avg_speed']
        if cb :
            value = "%.3f km/h" % value
            log( "avg_speed value=%s kw=%s" % (value,kw) )
            return f.set(self, value, **kw)
        if self.isKwStatus( kw, '_initializing_' ) :
            log( "avg_speed value=%s kw=%s" % (value,kw) )
            return f.set(self, value, **kw)
        return None

    security.declarePrivate('setMax_altitude')
    def setMax_altitude(self, value, cb=None, **kw):
        """set the MAX altitude"""

        f=self.Schema()['max_altitude']
        if cb :
            value = "%d m" % value
            log( "max_altitude value=%s kw=%s" % (value,kw) )
            return f.set(self, value, **kw)
        if self.isKwStatus( kw, '_initializing_' ) :
            log( "max_altitude value=%s kw=%s" % (value,kw) )
            return f.set(self, value, **kw)
        return None

    security.declarePrivate('setMin_altitude')
    def setMin_altitude(self, value, cb=None, **kw):
        """set the MIN altitude"""

        f=self.Schema()['min_altitude']
        if cb :
            value = "%d m" % value
            log( "min_altitude value=%s kw=%s" % (value,kw) )
            return f.set(self, value, **kw)
        if self.isKwStatus( kw, '_initializing_' ) :
            log( "min_altitude value=%s kw=%s" % (value,kw) )
            return f.set(self, value, **kw)
        return None

    security.declarePrivate('setSpeed_graph_image')
    def setSpeed_graph_image(self, value, cb=None, **kw):
        """set the speed graph image(png)"""

        if cb or self.isKwStatus( kw, '_initializing_' ) :
            f=self.Schema()['speed_graph_image']
            log( "speed_graph_image value=%s kw=%s" % (value,kw) )
            return f.set(self, value, **kw)
        return None

    security.declarePrivate('setAltitude_graph_image')
    def setAltitude_graph_image(self, value, cb=None, **kw):
        """set the alititude graph image(png)"""

        if cb or self.isKwStatus( kw, '_initializing_' ) :
            f=self.Schema()['altitude_graph_image']
            log( "altitude_graph_image value=%s kw=%s" % (value,kw) )
            return f.set(self, value, **kw)
        return None

    security.declarePrivate('setGpxlogfile')
    def setGpxlogfile(self, value, **kw):

        rtn = None
        # call fields setter
        f=self.Schema()['gpxlogfile']
        rtn =f.set(self, value, **kw)

        # when gpx file is uploaded, analyze file contents
        if not self.isKwStatus( kw, '_initializing_' ) :
            #Analyze GPX Log File & Set Field(String,Image...etc)
            data = self.getGPXLogData( value, refresh=True )
            # set All Running Data
            self.setRunningData( data )
            rtn = None
        return rtn

    def isSpeedGraph(self):
        """speed graph image conditon"""
        f=self.Schema()['speed_graph_image']
        file = f.get(self)
        if isinstance( file, OFSImage ):
          if len(file.data) : return True
        return None

    def isAltitudeGraph(self):
        """speed graph image conditon"""
        f=self.Schema()['altitude_graph_image']
        file = f.get(self)
        if isinstance( file, OFSImage ):
          if len(file.data) : return True
        return None

    def getKLMLogData( self ):
        """get GPXLogData for klm file(element=coordinates)"""

        rtn = ""
        log_data = self.getGPXLogData( )

        if log_data == {} :
           return ""
        
        for trk in log_data['trk'] :
            rtn += "%s," % trk['lon']
            rtn += "%s" % trk['lat']
            if trk.has_key('ele') :
                rtn += ",%s \n" % trk['ele']
            else :
                rtn += " \n"
        return rtn

    def getGPXLogData( self, file=None, refresh=False ):
        """Get the LogData informations of the file
        
        The information is cached in _v_gpx_logdata
        """
        cache = '_gpx_logdata'
        
#        import pdb
#        pdb.set_trace()
        if refresh:
            setattr(self, cache, None)
        
        log_data = getattr(self, cache, None)
        
        if log_data is None or not isinstance(log_data, dict):
            io = self.getLogFileAsFile( file )
            if io is not None:
                gpx = GPXAnalyze( )
                log_data = gpx.makeLogData( io )
                if log_data == {}:
                    log('Failed to process GPX Log Data information')
                    io.seek(0)

        if not log_data:
            # alawys return a dict
            log_data = {}
        # set the GPXLogData cache even if the image has returned an empty
        # dict. This prevents regenerating the exif every time if an
        # image doesn't have exif information.
        setattr(self, cache, log_data)
        return log_data

    def getLogFileAsFile( self, lf=None ):
        """Get the img as file like object
        """
        if lf is None:
            f = self.getField('gpxlogfile')
            lf = f.get(self)
        # img.data contains the image as string or Pdata chain
        data = None
        log("lf=%s",lf)
        if isinstance(lf,OFSFile):
            log('OFSFile')
            data = str(lf.data)
        if isinstance(lf, Pdata):
            log('PData')
            data = str(lf)
        elif isinstance(lf, str):
            log('str')
            data = lf
        elif isinstance(lf, file) or (hasattr(lf, 'read') and
          hasattr(lf, 'seek')):
            log('IO')
            lf.seek(0)
            return lf
        if data:
            return StringIO(data)
        else:
            return None


    security.declarePrivate('setRunningData')
    def setRunningData( self, data ):
        
        try :
            self.setTrk_log_count( len(data['trk']), cb=True )
        except KeyError, s:
            self.setTrk_log_count( 0 , cb=True )
            log("setTrk_log_count is None : %s" % s)
        try :
            self.setTotal_distance( data['dist_diff_data']['total'], cb=True )
        except KeyError, s:
            kw = {'_initializing_':True,'field':'total_distance','mimetype':'text/plain'}
            self.setTotal_distance( 'No Data', **kw )
            log("setTotal_distance is None : %s" % s)
        try :
            self.setStart_datetime( data['time_diff_data']['min'], cb=True )
        except KeyError, s:
            kw = {'_initializing_':True,'field':'start_datetime','mimetype':'text/plain'}
            self.setStart_datetime( 'No Data', **kw )
            log("setStart_datetime is None : %s" % s)
        try :
            self.setEnd_datetime( data['time_diff_data']['max'], cb=True )
        except KeyError, s:
            kw = {'_initializing_':True,'field':'end_datetime','mimetype':'text/plain'}
            self.setEnd_datetime( 'No Data', **kw )
            log("setEnd_datetime is None : %s" % s)
        try :
            self.setMoveing_time( data['time_diff_data']['total'], cb=True )
        except KeyError, s:
            kw = {'_initializing_':True,'field':'moveing_time','mimetype':'text/plain'}
            self.setMoveing_time( 'No Data', **kw )
            log("setMoveing_time is None : %s" % s)
        try :
            self.setMax_speed( data['speed_data']['max'], cb=True )
        except KeyError, s:
            kw = {'_initializing_':True,'field':'max_speed','mimetype':'text/plain'}
            self.setMax_speed( 'No Data', **kw )
            log("setMax_speed is None : %s" % s)
        try :
            self.setAvg_speed( data['speed_data']['avg'], cb=True )
        except KeyError, s:
            kw = {'_initializing_':True,'field':'avg_speed','mimetype':'text/plain'}
            self.setAvg_speed( 'No Data', **kw )
            log("setAvg_speed is None : %s" % s)
        try :
            self.setMax_altitude( data['ele_data']['max'], cb=True )
        except KeyError, s:
            kw = {'_initializing_':True,'field':'max_altitude','mimetype':'text/plain'}
            self.setMax_altitude( 'No Data', **kw )
            log("setMax_altitude is None : %s" % s)
        try :
            self.setMin_altitude( data['ele_data']['min'], cb=True )
        except KeyError, s:
            kw = {'_initializing_':True,'field':'min_altitude','mimetype':'text/plain'}
            self.setMin_altitude( 'No Data', **kw )
            log("setMin_altitude is None : %s" % s)

        # create & set Graph Images
        saveformat ="PNG"
        mimetype = 'image/%s' % saveformat.lower()
        # create altitude graph image
        self.createAltitudeGraph( data, saveformat, mimetype )
        # create speed graph image
        self.createSpeedGraph( data, saveformat, mimetype )

    security.declarePrivate('createAltitudeGraph')
    def createAltitudeGraph( self, data, format, mimetype ):
        
        newimg = StringIO()
        dist = []
        ele  = []
        cg = CreateGraph( mode="ele" )
        try :
            for i in data['trk'] :
                dist.append( i['dist_diff'] )
                ele.append( i['ele'] )
            cg.makeGraphBase( 0, data['dist_diff_data']['total'], data['ele_data']['min'], data['ele_data']['max'] )
            cg.drawLineGraph( dist, ele )
        except KeyError, s:
            log("AltitudeGraph is None : %s" % s)
        cg.saveGraphImage( newimg, format )
        fname = "altitude_graph_image." + format.lower()
        self.setAltitude_graph_image( newimg, cb=True, mimetype=mimetype,filename=fname )
        del cg
        newimg.close()

    security.declarePrivate('createSpeedGraph')
    def createSpeedGraph( self, data, format, mimetype ):
        
        newimg = StringIO()
        dist = []
        speed  = []
        cg = CreateGraph( mode="speed")
        try :
            for i in data['trk'] :
                dist.append( i['dist_diff'] )
                speed.append( i['speed'] )
            cg.makeGraphBase( 0, data['dist_diff_data']['total'], 0, data['speed_data']['max'] )
            cg.drawLineGraph( dist, speed )
        except KeyError, s:
            log("SpeedGraph is None : %s" % s)
        cg.saveGraphImage( newimg, format )
        fname = "speed_graph_image." + format.lower()
        self.setSpeed_graph_image( newimg, cb=True, mimetype=mimetype,filename=fname )
        del cg
        newimg.close()

    security.declarePrivate('isKwStatus')
    def isKwStatus( self, kw, key ):
        if kw.has_key( key ) :
            if kw[key] :
                return True
        return False

registerType(GGPXPolyline, PROJECTNAME)
