"""A Google GPolyline type"""

__author__  = '''takanori <takanori@takanory.net>'''
__docformat__ = 'plaintext'

from AccessControl import ClassSecurityInfo

try:
  from Products.LinguaPlone.public import *
except ImportError:
  # No multilingual support
  from Products.Archetypes.public import *

from Products.ATContentTypes.content.base import ATCTContent
from Products.ATContentTypes.content.schemata import ATContentTypeSchema
from Products.ATContentTypes.content.schemata import finalizeATCTSchema
from Products.ATContentTypes.lib.historyaware import HistoryAwareMixin

from Products.CMFPlone.interfaces.NonStructuralFolder import INonStructuralFolder

#from Products.Archetypes.atapi import *

from Products.ATGoogleMaps.config import *
from Products.ATGoogleMaps.field import LatLngField
from Products.ATGoogleMaps.widget import LatLngWidget

GPolylineSchema = ATContentTypeSchema.copy() + Schema((
    LinesField('points',
        widget=LinesWidget(
            label='Points',
            label_msgid='label_points',
            description='Creates a polyline from an array of points.',
            description_msgid='help_points',
            i18n_domain='googlemaps',
            rows=10,
            cols=30,
        )
    ),
    
    StringField('color',
        widget=StringWidget(
            label='Color',
            label_msgid='label_color',
            description='Enter the color in hexadecimal numeric HTML style, i.e. #RRGGBB.',
            description_msgid='help_color',
            i18n_domain='googlemaps',
            size=10,
        )
    ),

    IntegerField('width',
        default=10,
        vocabulary=range(1,20),
        widget=SelectionWidget(
            label='Width',
            label_msgid='label_width',
            description='Select a width of the line in pixels.',
            description_msgid='help_width',            
            i18n_domain='googlemaps',
        )
    ),
    
    BooleanField('opacity',
        widget=BooleanWidget(
            label='Opacity',
            label_msgid='label_opacity',
            description_msgid='help_opacity',
            i18n_domain='googlemaps',
        )
    ),
    
),
)

#Finalize schema definition
finalizeATCTSchema(GPolylineSchema)

class GPolyline(ATCTContent, HistoryAwareMixin):
    """Drow polyline in Google Maps"""
    
    schema          = GPolylineSchema

    content_icon    = 'gpolyline_icon.gif'
    portal_type     = 'GPolyline'
    meta_type       = 'GPolyline'
    archetype_name  = 'GPolyline'
    #default_view    = 'gpolyline_view'
    #immediate_view  = 'gpolyline_view'
    suppl_views     = ()
    typeDescription = 'Drow polyline in Google Maps'
    typeDescMsgId   = 'description_edit_gpolyline'

    global_allow    = 0

    # Get the standard actions (tabs)
    #actions = ATCTContent.actions

    # Make sure we get title-to-id generation when an object is created
    _at_rename_after_creation = True

registerType(GPolyline, PROJECTNAME)
