import PIL
import ImageDraw
import math

class CreateGraph :

    def __init__(self, mode, size=None, basecol=None, fontcol=None,
                 gcol=None, format='PNG', debug=0):

        self.offset  = 30
        self.size    = (600,400)      # graph field size
        self.bgcol   = (255,255,255)  # graph field background color
        self.basecol = (237,234,229)  # backbround color
        self.fontcol = (137,0,  0  )  # font color
        self.gcol    = (221,0,  0  )  # line-graph color
        self.y_col   = (0,  0,  0  )  # Yscale line color(graph field outline)
        self.x_col   = (216,216,216)  # Xscale line color
        self.neg_col = (240,240,240)  # Negative field background color
        self.im      = None
        self.debug   = debug
        self.x_unit  = {}
        self.y_unit  = {}
        self.mode    = mode
        self.format  = format
        self.zero_abs= [0.0,0.0]

        if size and isinstance(size,tuple) :
            self.size = size
        if basecol :
            col = self.chgColorType( basecol )
            if col : self.basecol = basecol
        if fontcol :
            col = self.chgColorType( fontcol )
            if col : self.fontcol = fontcol
        if gcol :
            col = self.chgColorType( gcol )
            if col : self.gcol = gcol

        size = (self.size[0]+self.offset*2,self.size[1]+self.offset*2)
        self.im = PIL.Image.new( "RGB", size, self.basecol )


    def chgColorType( self, col ) :
        if isinstance( col, tuple ) and len(col) == 3 :
            return col
        if isinstance( col, str ) :
            try:
                col = PIL.ImageColor.getrgb( col )
            except ValueError, e :
                col = None
            return col
        return None


    def getFontImages( self ) :
        
        # all font image data ...orz
        polygon = [ \
            # 0
            [ \
                ['0','p','f',[ (0,0),(4,0),(4,7),(0,7) ] ], \
                ['0','l','b',[ (2,1),(2,6)] ] \
            ], \
            # 1
            [ \
                ['1','l','f',[ (1,0),(3,0) ] ], \
                ['1','p','f',[ (2,0),(3,0),(3,7),(2,7) ] ] \
            ], \
            # 2
            [ \
                ['2','p','f',[ (0,0),(4,0),(4,7),(0,7) ] ], \
                ['2','p','b',[ (0,1),(2,1),(2,3),(0,3) ] ], \
                ['2','p','b',[ (2,4),(4,4),(4,6),(2,6) ] ] \
            ], \
            # 3
            [ \
                ['3','p','f',[ (0,0),(4,0),(4,7),(0,7) ] ], \
                ['3','p','b',[ (0,1),(2,1),(2,6),(0,6) ] ], \
                ['3','l','f',[ (1,3),(4,3) ] ] \
            ], \
            # 4
            [ \
                ['4','p','f',[ (0,0),(4,0),(4,7),(0,7) ] ], \
                ['4','p','b',[ (0,4),(2,4),(2,7),(0,7) ] ], \
                ['4','l','b',[ (2,0),(2,3) ] ] \
            ], \
            # 5
            [ \
                ['5','p','f',[ (0,0),(4,0),(4,7),(0,7) ] ], \
                ['5','p','b',[ (2,1),(4,1),(4,3),(2,3) ] ], \
                ['5','p','b',[ (0,4),(2,4),(2,6),(0,6) ] ] \
            ], \
            # 6
            [ \
                ['6','p','f',[ (0,0),(4,0),(4,7),(0,7) ] ], \
                ['6','p','b',[ (2,1),(4,1),(4,3),(2,3) ] ], \
                ['6','l','b',[ (2,4),(2,6) ] ] \
            ], \
            # 7
            [ \
                ['7','l','f',[ (0,0),(4,0) ] ], \
                ['7','p','f',[ (3,0),(4,0),(4,7),(3,7) ] ] \
            ], \
            # 8
            [ \
                ['8','p','f',[ (0,0),(4,0),(4,7),(0,7) ] ], \
                ['8','l','b',[ (2,1),(2,3) ] ], \
                ['8','l','b',[ (2,4),(2,6) ] ] \
            ], \
            # 9
            [ \
                ['9','p','f',[ (0,0),(4,0),(4,7),(0,7) ] ], \
                ['9','l','b',[ (2,1),(2,3) ] ], \
                ['9','p','b',[ (0,4),(2,4),(2,6),(0,6) ] ] \
            ], \
            # -
            [ \
                ['-','p','f',[ (0,3),(4,3),(4,4),(0,4) ] ] \
            ], \
            # k
            [ \
                ['k','l','f',[ (0,0),(0,7) ] ], \
                ['k','l','f',[ (4,1),(1,4) ] ], \
                ['k','l','f',[ (1,4),(4,7) ] ], \
                ['k','l','f',[ (3,6),(4,6) ] ] \
            ], \
            # m
            [ \
                ['m','p','f',[ (0,2),(4,2),(4,7),(0,7) ] ], \
                ['m','l','b',[ (4,2),(5,2) ] ], \
                ['m','l','b',[ (1,3),(1,7) ] ], \
                ['m','l','b',[ (3,3),(3,7) ] ] \
            ], \
            # /
            [ \
                ['/','l','f',[ (4,0),(0,7) ] ] \
            ], \
            # h
            [ \
                ['h','l','f',[ (0,0),(0,7) ] ], \
                ['h','l','f',[ (0,3),(1,3) ] ], \
                ['h','l','f',[ (2,2),(3,2) ] ], \
                ['h','l','f',[ (4,3),(4,7) ] ] \
            ], \
            # [
            [ \
                ['[','l','f',[ (1,0),(3,0) ] ], \
                ['[','l','f',[ (1,0),(1,7) ] ], \
                ['[','l','f',[ (1,6),(3,6) ] ] \
            ], \
            # ]
            [ \
                [']','l','f',[ (1,0),(3,0) ] ], \
                [']','l','f',[ (3,0),(3,7) ] ], \
                [']','l','f',[ (1,6),(3,6) ] ] \
            ], \
        ]
        imglist = {}
        
        size = ( 6,8 )
        imglist['size'] = size

        for font in polygon :
            img = PIL.Image.new( "RGB", size, self.basecol )
            draw  = ImageDraw.Draw(img)
            key = font[0][0]
            for l in font :
                col = self.fontcol
                if l[2] == 'b' : col = self.basecol
                if l[1] == 'p' :
                    draw.polygon( l[3], fill=col )
                elif l[1] == 'l' :
                    draw.line( l[3], fill=col )
            imglist[key] = img.crop( ( 0, 0, size[0], size[1]) )
            del draw
            del img

        return imglist


    def getGraphBaseSquare( self, offset=0 ) :

        sq = []
        sq.append( (self.offset-offset,              self.offset-offset ) )
        sq.append( (self.size[0]+self.offset+offset, self.offset-offset ) )
        sq.append( (self.size[0]+self.offset+offset,
                                         self.size[1]+self.offset+offset) )
        sq.append( (self.offset-offset,
                                         self.size[1]+self.offset+offset) )
        if self.debug : print "getGraphBaseSquare(%d) : %s" % ( offset, sq )
        return sq


    def disctionLineUnit( self, width, minls, maxv, disits ) :

        # width   = グラフエリアの最大幅(px)
        # minls   = ラインの最小間隔(px)
        # maxv    = 距離とかの最大値(メートル)
        # disits  = (list)最大桁リスト(10,100,1000,10000..) これも単位はm
        
        line_dist = []
        rtn_unit = {}

        f_width = float(width)
        f_maxv  = float(maxv)
        
        # graph max width(height) : ? px = max value : 1
        px_par_val = f_width / f_maxv
        
        lp = f_width / minls
#        if width % minls : lp += 1
        for i in disits :
            for j in (1,2,5) :
                line_dist.append( {'lval': i*j, 'ldigit': i,
                                   'lstep': j, 'maxval': i*j*lp} )
        
        for i in line_dist :
            if i['maxval'] >= f_maxv :
                rtn_unit = i
                break
        # 1value当たりのpx
        if self.debug : print rtn_unit
        rtn_unit['px_par_val']= px_par_val

        # m or km ?
        if rtn_unit['ldigit'] >= 1000 :
            rtn_unit['lunit']  = 'km'
            rtn_unit['ldigit'] = rtn_unit['ldigit'] / 1000
        else :
            rtn_unit['lunit'] = 'm'

        del(rtn_unit['maxval'])
        del(rtn_unit['lval'])
        
        if self.debug : print rtn_unit
        return rtn_unit


    def makeGraphBase(self, x_min, x_max, y_min, y_max ) :
        # x = dist, y = speed or altitude
        
        draw  = ImageDraw.Draw(self.im)
        nfont = self.getFontImages( )
        imsize = nfont['size']

        # graph base square
        basebox = self.getGraphBaseSquare( 1 )
        draw.polygon( basebox, fill=self.y_col)
        basebox = self.getGraphBaseSquare( 0 )
        draw.polygon( basebox, fill=self.bgcol)

        org_xy = ( basebox[0][0], basebox[0][1] )
        max_xy = ( basebox[2][0], basebox[2][1] )

        #######################################################################
        # X軸(dist[m or km]) の目盛単位を決める
        #                                      (max=10000kmは対応してると思う)
        #   X軸ではマイナス値はないと決め打ち(x_minの意味ねぇえええ)

        # 最小ライン幅決定(=fontの縦サイズ×千桁(4))
        # default = 6px * 4 = 24px
        min_ls_x = imsize[0] * 4 + 8
        min_ls_x += 8  #ちょっと調整

        self.x_unit = self.disctionLineUnit( self.size[0], min_ls_x,
                                       x_max, (10,100,1000,10000,100000) )
        
        # X軸の-方向側の絶対値(pxではない)。Xは0固定
        self.zero_abs[0] = 0.0
        x_str_int = 0
        x_px = 0
        x_unit_offset = 1000
        if self.x_unit['lunit'] == 'm' : x_unit_offset = 1

        #######################################################################
        # Y軸(alt or speed[m/km or km/h]) の目盛単位を決める
        #                                      (max=10000kmは対応してると思う)
        #   Y軸は-方向にも対応。speedはないと思うが、altitudeが・・・
        
        # 最小ライン幅決定(=fontの縦サイズ)
        # default = 8px
        min_ls_y = imsize[1]
        min_ls_y += 13  #ちょっと調整

        # 最大と最小が+方向か-方向だったらどっちかを0に張り付かせる
        if y_min < 0 and y_max < 0 : y_max = 0
        if y_min > 0 and y_max > 0 : y_min = 0
        y_gap = y_max - y_min

        if self.mode == "speed" :
            if self.debug : print "mode speed graph"
            # 速度は%.fkm/hなので小数点以下丸め込みの上、メートルに変換して渡す
            self.y_unit = self.disctionLineUnit( self.size[1], min_ls_y,
                                           int(y_gap) * 1000, (10000,100000) )
        else :
            if self.debug : print "mode elevation graph"
            # 高度はmなのでそのまま
            self.y_unit = self.disctionLineUnit( self.size[1], min_ls_y,
                                           y_gap, (10,100,1000,10000,100000) )
        
        y_str_int = 0
        y_px = 0
        y_unit_offset = 1000
        # X軸の-方向側の絶対値(pxではない)。これでGraph上の0の位置を底上げ
        self.zero_abs[1] = abs(y_min)
        if self.y_unit['lunit'] == 'm' : y_unit_offset = 1
        if y_min < 0 :
            u = float(self.y_unit['ldigit']) * float(self.y_unit['lstep'])
            d = math.modf( float(y_min) / u )
            f = math.fmod( float(y_min), u )
            y_str_int = int( d[1] * u )
            y_px = int( abs(f * self.y_unit['px_par_val'] * y_unit_offset) )

#            import pdb
#            pdb.set_trace()

            # Y軸マイナス値エリアをちょい灰色に塗りつぶす
            y_0_px = int( abs(y_min*self.y_unit['px_par_val']*y_unit_offset) )
            sq = []
            sq.append( (org_xy[0], int( max_xy[1] - y_0_px )) )
            sq.append( (max_xy[0], int( max_xy[1] - y_0_px )) )
            sq.append( (max_xy[0], max_xy[1]                ) )
            sq.append( (org_xy[0], max_xy[1]                ) )
            draw.polygon( sq, fill=self.neg_col)

        #######################################################################
        # X軸目盛ライン＆文字描画処理

        while x_px < self.size[0] :
            x = int( org_xy[0] + (x_px+0.5) )
            draw.line( [(x,org_xy[1]),(x,max_xy[1])], fill=self.x_col )
            fontstr = "%s" % x_str_int
            for i in range(0,len(fontstr)) :
                if self.debug : print "%s %d" % (fontstr,eval(fontstr[i]))
                box = (x+(i*imsize[0]), max_xy[1]+3,
                       x+(i*imsize[0])+imsize[0], max_xy[1]+3+imsize[1])
                self.im.paste( nfont[fontstr[i]], box )
            x_str_int = x_str_int + \
                           (self.x_unit['ldigit'] * self.x_unit['lstep'])
            x_px      = x_str_int * \
                            self.x_unit['px_par_val'] * x_unit_offset

        # put unit string [m] or [km]
        f_org_x = max_xy[0] + 3
        f_org_y = max_xy[1] + 3 + imsize[1]
        st_font = '[' + self.x_unit['lunit'] + ']'
        for i in range( 0,len( st_font ) ) :
            box = (f_org_x + (i*imsize[0]), f_org_y,
                   f_org_x + (i*imsize[0]) + imsize[0], f_org_y + imsize[1])
            self.im.paste( nfont[st_font[i]], box )

        #######################################################################
        # Y軸目盛ライン＆文字描画処理

        while y_px < self.size[1] :
            y = int( max_xy[1] - y_px )
            draw.line( [(org_xy[0],y),(max_xy[0],y)],fill=self.y_col)
            fontstr = "%s" % y_str_int
            if self.debug : print len(fontstr)
            for i in range(0,len(fontstr)) :
                offset = len(fontstr) - i - 1
                if self.debug : print "%s %d" % (fontstr,eval(fontstr[i]))
                f_org_x = org_xy[0] - 3 - offset*imsize[0] - imsize[0]
                f_org_y = y - imsize[1]
                box = ( f_org_x, f_org_y, f_org_x + imsize[0],
                        f_org_y + imsize[1] )
                self.im.paste( nfont[fontstr[i]], box )
            u = self.y_unit['ldigit'] * self.y_unit['lstep']
            y_str_int = y_str_int + u
            y_px = y_px + ( u * self.y_unit['px_par_val'] * y_unit_offset )

        # put unit string [km/h]
        f_org_x = org_xy[0] - imsize[0]*3 - 3
        f_org_y = org_xy[1] - 3 - (imsize[1]*2)
        if self.mode == "speed" :
            st_font = '[km/h]'
        else :
            st_font = '[' + self.y_unit['lunit'] + ']'
        
        for i in range( 0,len( st_font ) ) :
            box = (f_org_x + (i*imsize[0]), f_org_y,
                   f_org_x + (i*imsize[0]) + imsize[0], f_org_y+imsize[1] )
            self.im.paste( nfont[st_font[i]], box )


    def drawLineGraph( self , x_list, y_list ) :

        draw  = ImageDraw.Draw(self.im)
        basebox = self.getGraphBaseSquare( 0 )
        
        org_xy = ( basebox[3][0], basebox[3][1] )
        max_xy = ( basebox[1][0], basebox[1][1] )
        
        # x = distance / y = speed
        # x = distance / y = ele
        x_all = 0
        
        x_par = self.x_unit['px_par_val']
        y_par = self.y_unit['px_par_val']

        unit_offset = 1
        if self.mode == "speed" : unit_offset = 1000
        if self.debug : print "basebox : %s %s" % (org_xy, max_xy)
        if self.debug : print "1px/x(m)=%f 1px/y(m)=%f" % (x_par,y_par)
        for i in range( 1, len(x_list) ) :
            f_x_1  = float(x_list[i-1])
            f_y_1 = float(y_list[i-1])
            f_x_2  = float(x_list[i])
            f_y_2 = float(y_list[i])

            x_px1 = int(x_par * (x_all + self.zero_abs[0]))
            y_px1 = int(y_par * ((f_y_1 + self.zero_abs[1]) * unit_offset))

            x_px2 = int(x_par * (x_all + f_x_2 + self.zero_abs[0]))
            y_px2 = int(y_par * ((f_y_2 + self.zero_abs[1]) * unit_offset))

            xy = [( org_xy[0] + x_px1, org_xy[1] - y_px1),\
                  ( org_xy[0] + x_px2, org_xy[1] - y_px2)]
            if self.debug :
                print "dist1=%d,dist2=%d" % (x_all, x_all+f_x_2)
            if self.debug :
                print "x=(%.3f->%.3f y=%.3f->%.3f (%d,%d -> %d,%d)" % \
                       (f_x_1,f_x_2,f_y_1,f_y_2,x_px1,y_px1,x_px2,y_px2)
            draw.line( xy, fill=self.gcol)
            x_all += f_x_2


    def saveGraphImage( self, save, format ):
        self.im.save( save, format )


    def getGraphImage( self ):
        return self.im
