from datetime import tzinfo, timedelta, datetime
import time
import math

class UTC(tzinfo):
    """UTC"""
    def utcoffset(self, dt):
        return timedelta(0)

    def tzname(self, dt):
        return "UTC"

    def dst(self, dt):
        return timedelta(0)

class JST(tzinfo):
    """JST"""
    def utcoffset(self, dt):
        return timedelta(hours=9)

    def tzname(self, dt):
        return "JST"

    def dst(self, dt):
        return timedelta(hours=9)

class GPSDataConv :

    def convUTCstr2JST( self, s, fmt ) :
        # GPXにある時刻はUTCと決めつけて処理
        # UTC -> JST 変換はこの辺を参照してtestしまくった...orz
        # http://www.python.jp/doc/2.4/lib/datetime-datetime.html
        # http://www.python.jp/doc/2.4/lib/datetime-tzinfo.html
        utc = UTC()
        jst = JST()
        
        jt = ''
#        import pdb
#        pdb.set_trace()
        try :
            t = time.strptime( s, fmt )
            ut = datetime( year=t[0],month=t[1],day=t[2], hour=t[3], minute=t[4], second=t[5], tzinfo=utc )
        except :
            return None

        return ut.astimezone(jst)

    def chgDegreeToRadian( self, data ) :
        return (data / 360 * 2 * 3.14159265)

    def getDistance( self, lat1, lon1, lat2, lon2 ) :
        # http://wadati.blog10.fc2.com/blog-entry-345.html
        a = 6378137

        if isinstance(lat1,str) or isinstance(lat1,unicode) : lat1 = eval( lat1 )
        lat1 = self.chgDegreeToRadian( lat1 )
        if isinstance(lat2,str) or isinstance(lat2,unicode) : lat2 = eval( lat2 )
        lat2 = self.chgDegreeToRadian( lat2 )
        if isinstance(lon1,str) or isinstance(lon1,unicode) : lon1 = eval( lon1 )
        lon1 = self.chgDegreeToRadian( lon1 )
        if isinstance(lon2,str) or isinstance(lon2,unicode) : lon2 = eval( lon2 )
        lon2 = self.chgDegreeToRadian( lon2 )

        lat = lat2 - lat1
        lon = lon2 - lon1
        lat_x = a * lat * math.cos(lon1)
        lon_y = a * lon
        return math.sqrt( lat_x**2 + lon_y**2 )

    def getSpeedMPerM ( self, dist, seconds ) :
        return dist / seconds

    def getSpeedKmPerH ( self, dist, seconds ) :
        speed = self.getSpeedMPerM( dist, seconds )
        return (speed * 60 * 60 /1000)
