from Products.ATGoogleMaps.lib.gpsdata import *
from xml.dom.ext.reader.Sax import FromXmlStream
from datetime import timedelta

##########################################################################
#
#  *.gpx file Class 
#    ・<trkseg>は無視でfile内にある<trkpt>をいっしょくたに扱ってたりする
#    ・<trk>が複数出てもいっしょくた(多分)
#    ・<wpt>は・・・どうしよーか
#
##########################################################################

class GPXAnalyze( GPSDataConv ):

    def __init__(self, debug=0):
        self.gpx_data = {}
        self.debug = debug
        self.item_list = {'trk':['lon', 'lat', 'ele', 'time'],\
                          'wpt':['lon', 'lat', 'ele', 'time', 'cmt']}

    def makeItemListStr( self, list ) :
        s = "{"
        for i in list :
            s += "'%s':''," % i
        s += "}"
        return s

    def initTrkItems( self, name ) :
        
        s = self.makeItemListStr( self.item_list[name] )
        self.gpx_data[name].append(eval(s))


    def setTrkItemList( self, name, value ):
        
        item = value
        if name == 'time' : item = self.convUTCstr2JST( value, "%Y-%m-%dT%H:%M:%SZ" )
        now_trkpt = len(self.gpx_data['trk']) - 1
        self.gpx_data['trk'][now_trkpt][name] = item
        if self.debug: print "%s(%d)=%s" % (name,len(self.gpx_data['trk']), item)


    def setTrkItemValue( self, name, value ):
        self.gpx_data[name] = value
        if self.debug: print "%s=%s\n" % (name, value)


    def getNodeAttr( self, node ):
        name  = node.nodeName
        attrs = node.attributes
        if attrs != None and len(attrs) > 0:
            for a in attrs:
                if name == 'trkpt' and (a.name == 'lat' or a.name == 'lon'):
                    self.setTrkItemList( a.name, a.value )
                if name == 'gpx' and a.name == 'creator':
                    self.setTrkItemValue( a.name, a.value )


    def nodeInspect( self, node, pname='' ):
        name = node.nodeName
        value = node.nodeValue

        if name == 'gpx':
            self.getNodeAttr( node )
        elif name == 'trk' :
            self.gpx_data['trk'] = []
        elif name == 'trkpt' :
            self.initTrkItems( 'trk' )

        if pname == "name" :
            self.setTrkItemValue( pname, value )
        elif pname == "trkseg":
            self.getNodeAttr( node )
        elif pname in ('ele','time') :
            self.setTrkItemList( pname, value )

        children = node.childNodes
        for c in children:
          self.nodeInspect( c, name )


    def checkTrkData( self, trk_data ) :
    
        # time check (all none ?)
        timef = 0
        for j in trk_data :
            if j['time'] != '' :
                timef = 1
                break
        
        if timef == 0 :
            for j in trk_data :
                del( j['time'] )

        elef  = 0
        for j in trk_data :
            if j['ele'] != '' :
                elef = 1
                break
        if elef == 0 :
            for j in trk_data :
                del( j['ele'] )

        dellist = []
        for i in range( len(trk_data) ) :
            if   trk_data[i]['lon'] == '' or eval(trk_data[i]['lon']) == 0 :
                dellist.append(i)
            elif trk_data[i]['lat'] == '' or eval(trk_data[i]['lat']) == 0 :
                dellist.append(i)
            elif trk_data[i].has_key('time') :
                if trk_data[i]['time'] == ''  or None :
                    dellist.append(i)
            elif trk_data[i].has_key('ele') :
                if trk_data[i]['ele']  == '' :
                    dellist.append(i)
        
        dellist.reverse()
        for i in dellist :
            del( trk_data[i] )
        if self.debug and len(dellist) : print "checkTrkData() : illegal data found"

    def getMaxMinAvgTotal( self, list, target ) :
        total = timedelta(0)
        rtn = {}
        l_list = []
        is_time = 0

        #print type(list[0])
        if type( list[0][target] ) == type(total) :
            is_time = 1
        else :
            total = 0.0

        for i in list :
            l_list.append( i[target] )
            total += i[target]
        
        if is_time == 1 :
            rtn['min'] = list[0]['time']
            rtn['max'] = list[len(list)-1]['time']
        else :
            l_list.sort()
            l_list.reverse()
            rtn['max']   = l_list[0]
            rtn['min']   = l_list[len(l_list)-1]
            rtn['avg']   = total / len(l_list)
            
        rtn['total'] = total

        return rtn

    def addRunningData( self ) :
        
        trk_list = self.gpx_data['trk']

        trk_former = trk_list[0]
        for trk in trk_list :
            dist_diff = self.getDistance( trk_former['lat'], trk_former['lon'],
                                          trk['lat'], trk['lon'] )
            trk['dist_diff'] = dist_diff

            if trk_list[0].has_key( 'time' ) :
                time_diff = trk['time'] - trk_former['time']
                speed = 0.0
                if time_diff != timedelta(0) :
                    speed = self.getSpeedKmPerH( dist_diff, time_diff.seconds )
                trk['speed'] = speed
                trk['time_diff'] = time_diff
            
            if trk.has_key( 'ele' ) :
                ele = eval(trk['ele'])
                #if ele < 0 : ele = 0
                trk['ele'] = ele
            
            trk_former = trk
        
        #make max,min,avg,total data
        data = ['dist_diff','time_diff','speed','ele']
        for tgt in data :
            if trk_list[0].has_key( tgt ) :
                self.gpx_data[tgt+'_data'] = self.getMaxMinAvgTotal( trk_list, tgt )
                if self.debug : print "%s : %s" % (tgt, self.gpx_data[tgt+'_data'])

    def makeLogData( self, io ):

        try :
            io.seek(0)
            doc = FromXmlStream(io)
        except :
            if self.debug : print "No GPX Track Log File"
            return {}
        
        self.nodeInspect( doc )
        
        # data matching & check
        if self.gpx_data.has_key('trk') :
            self.checkTrkData( self.gpx_data['trk'] )
            if len( self.gpx_data['trk'] ) != 0 :
                self.addRunningData( )
            else :
                self.gpx_data = {}
        else :
            self.gpx_data = {}

        return self.gpx_data

